function b2Mat33(c1, c2, c3) {
	this.col1 = new b2Vec3();
	this.col2 = new b2Vec3();
	this.col3 = new b2Vec3();
	if (c1) {
		this.col1.SetV(c1);
	}
	if (c2) {
		this.col2.SetV(c2);
	}
	if (c3) {
		this.col3.SetV(c3);
	}
}

Box2D.b2Mat33 = b2Mat33;

b2Mat33.prototype = {
	SetVVV : function (c1, c2, c3) {
		this.col1.SetV(c1);
		this.col2.SetV(c2);
		this.col3.SetV(c3);
	},

	Copy : function () {
		return new b2Mat33(this.col1, this.col2, this.col3);
	},

	SetM : function (m) {
		this.col1.SetV(m.col1);
		this.col2.SetV(m.col2);
		this.col3.SetV(m.col3);
	},

	AddM : function (m) {
		this.col1.x += m.col1.x;
		this.col1.y += m.col1.y;
		this.col1.z += m.col1.z;
		this.col2.x += m.col2.x;
		this.col2.y += m.col2.y;
		this.col2.z += m.col2.z;
		this.col3.x += m.col3.x;
		this.col3.y += m.col3.y;
		this.col3.z += m.col3.z;
	},

	SetIdentity : function () {
		this.col1.Set(1, 0, 0);
		this.col2.Set(0, 1, 0);
		this.col3.Set(0, 0, 1);
	},

	SetZero : function () {
		this.col1.Set(0, 0, 0);
		this.col2.Set(0, 0, 0);
		this.col3.Set(0, 0, 0);
	},

	Solve22 : function (out, bX, bY) {
		var a11 = this.col1.x,
			a12 = this.col2.x,
			a21 = this.col1.y,
			a22 = this.col2.y,
			det = a11 * a22 - a12 * a21;

		bX = bX || 0;
		bY = bY || 0;

		if (det) {
			det = 1 / det;
		}

		out.x = det * (a22 * bX - a12 * bY);
		out.y = det * (a11 * bY - a21 * bX);

		return out;
	},

	Solve33 : function (out, bX, bY, bZ) {
		var a11 = this.col1.x,
			a21 = this.col1.y,
			a31 = this.col1.z,
			a12 = this.col2.x,
			a22 = this.col2.y,
			a32 = this.col2.z,
			a13 = this.col3.x,
			a23 = this.col3.y,
			a33 = this.col3.z,
			det = a11 * (a22 * a33 - a32 * a23) + a21 * (a32 * a13 - a12 * a33) + a31 * (a12 * a23 - a22 * a13);

		bX = bX || 0;
		bY = bY || 0;
		bZ = bZ || 0;

		if (det) {
			det = 1 / det;
		}

		out.x = det * (bX * (a22 * a33 - a32 * a23) + bY * (a32 * a13 - a12 * a33) + bZ * (a12 * a23 - a22 * a13));
		out.y = det * (a11 * (bY * a33 - bZ * a23) + a21 * (bZ * a13 - bX * a33) + a31 * (bX * a23 - bY * a13));
		out.z = det * (a11 * (a22 * bZ - a32 * bY) + a21 * (a32 * bX - a12 * bZ) + a31 * (a12 * bY - a22 * bX));

		return out;
	}
};
