(function () {
	var b2DebugDraw = Box2D.Dynamics.b2DebugDraw;
	b2DebugDraw.b2DebugDraw = function () {
		this.m_drawScale = 1.0;
		this.m_lineThickness = 1.0;
		this.m_alpha = 1.0;
		this.m_fillAlpha = 1.0;
		this.m_xformScale = 1.0;
		var __this = this;
		//#WORKAROUND
		this.m_sprite = {
			graphics: {
				clear: function () {
					__this.m_ctx.clearRect(0, 0, __this.m_ctx.canvas.width, __this.m_ctx.canvas.height)
				}
			}
		};
	};
	b2DebugDraw.prototype._color = function (color, alpha) {
		return "rgba(" + ((color & 0xFF0000) >> 16) + "," + ((color & 0xFF00) >> 8) + "," + (color & 0xFF) + "," + alpha + ")";
	};
	b2DebugDraw.prototype.SetLineThickness = function (lineThickness) {
		if (lineThickness === undefined) lineThickness = 0;
		this.m_lineThickness = lineThickness;
		this.m_ctx.strokeWidth = lineThickness;
	};
	b2DebugDraw.prototype.GetLineThickness = function () {
		return this.m_lineThickness;
	};
	b2DebugDraw.prototype.SetAlpha = function (alpha) {
		if (alpha === undefined) alpha = 0;
		this.m_alpha = alpha;
	};
	b2DebugDraw.prototype.GetAlpha = function () {
		return this.m_alpha;
	};
	b2DebugDraw.prototype.SetFillAlpha = function (alpha) {
		if (alpha === undefined) alpha = 0;
		this.m_fillAlpha = alpha;
	};
	b2DebugDraw.prototype.GetFillAlpha = function () {
		return this.m_fillAlpha;
	};
	b2DebugDraw.prototype.SetXFormScale = function (xformScale) {
		if (xformScale === undefined) xformScale = 0;
		this.m_xformScale = xformScale;
	};
	b2DebugDraw.prototype.GetXFormScale = function () {
		return this.m_xformScale;
	};
	b2DebugDraw.prototype.DrawPolygon = function (vertices, vertexCount, color) {
		if (!vertexCount) return;
		var s = this.m_ctx;
		var drawScale = this.m_drawScale;
		s.beginPath();
		s.strokeStyle = this._color(color.color, this.m_alpha);
		s.moveTo(vertices[0].x * drawScale, vertices[0].y * drawScale);
		for (var i = 1; i < vertexCount; i++) {
			s.lineTo(vertices[i].x * drawScale, vertices[i].y * drawScale);
		}
		s.lineTo(vertices[0].x * drawScale, vertices[0].y * drawScale);
		s.closePath();
		s.stroke();
	};
	b2DebugDraw.prototype.DrawSolidPolygon = function (vertices, vertexCount, color) {
		if (!vertexCount) return;
		var s = this.m_ctx;
		var drawScale = this.m_drawScale;
		s.beginPath();
		s.strokeStyle = this._color(color.color, this.m_alpha);
		s.fillStyle = this._color(color.color, this.m_fillAlpha);
		s.moveTo(vertices[0].x * drawScale, vertices[0].y * drawScale);
		for (var i = 1; i < vertexCount; i++) {
			s.lineTo(vertices[i].x * drawScale, vertices[i].y * drawScale);
		}
		s.lineTo(vertices[0].x * drawScale, vertices[0].y * drawScale);
		s.closePath();
		s.fill();
		s.stroke();
	};
	b2DebugDraw.prototype.DrawCircle = function (center, radius, color) {
		if (!radius) return;
		var s = this.m_ctx;
		var drawScale = this.m_drawScale;
		s.beginPath();
		s.strokeStyle = this._color(color.color, this.m_alpha);
		s.arc(center.x * drawScale, center.y * drawScale, radius * drawScale, 0, Math.PI * 2, true);
		s.closePath();
		s.stroke();
	};
	b2DebugDraw.prototype.DrawSolidCircle = function (center, radius, axis, color) {
		if (!radius) return;
		var s = this.m_ctx,
			drawScale = this.m_drawScale,
			cx = center.x * drawScale,
			cy = center.y * drawScale;
		s.moveTo(0, 0);
		s.beginPath();
		s.strokeStyle = this._color(color.color, this.m_alpha);
		s.fillStyle = this._color(color.color, this.m_fillAlpha);
		s.arc(cx, cy, radius * drawScale, 0, Math.PI * 2, true);
		s.moveTo(cx, cy);
		s.lineTo((center.x + axis.x * radius) * drawScale, (center.y + axis.y * radius) * drawScale);
		s.closePath();
		s.fill();
		s.stroke();
	};
	b2DebugDraw.prototype.DrawSegment = function (p1, p2, color) {
		var s = this.m_ctx,
			drawScale = this.m_drawScale;
		s.strokeStyle = this._color(color.color, this.m_alpha);
		s.beginPath();
		s.moveTo(p1.x * drawScale, p1.y * drawScale);
		s.lineTo(p2.x * drawScale, p2.y * drawScale);
		s.closePath();
		s.stroke();
	};
	b2DebugDraw.prototype.DrawTransform = function (xf) {
		var s = this.m_ctx,
			drawScale = this.m_drawScale;
		s.beginPath();
		s.strokeStyle = this._color(0xff0000, this.m_alpha);
		s.moveTo(xf.position.x * drawScale, xf.position.y * drawScale);
		s.lineTo((xf.position.x + this.m_xformScale * xf.R.col1.x) * drawScale, (xf.position.y + this.m_xformScale * xf.R.col1.y) * drawScale);

		s.strokeStyle = this._color(0xff00, this.m_alpha);
		s.moveTo(xf.position.x * drawScale, xf.position.y * drawScale);
		s.lineTo((xf.position.x + this.m_xformScale * xf.R.col2.x) * drawScale, (xf.position.y + this.m_xformScale * xf.R.col2.y) * drawScale);
		s.closePath();
		s.stroke();
	};
})();




